"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const UpdateChanges_1 = require("../common/UpdateChanges");
const util_1 = require("../common/util");
const ts = require("typescript");
const version = '18.2.0';
exports.default = () => (host, context) => __awaiter(void 0, void 0, void 0, function* () {
    context.logger.info(`Applying migration for Ignite UI for Angular to version ${version}`);
    const update = new UpdateChanges_1.UpdateChanges(__dirname, host, context);
    const changes = new Map();
    const oldProp = 'shouldGenerate';
    const newProp = 'autoGenerate';
    const addChange = (path, change) => {
        if (changes.has(path)) {
            changes.get(path).push(change);
        }
        else {
            changes.set(path, [change]);
        }
    };
    const applyChanges = () => {
        for (const [path, fileChanges] of changes.entries()) {
            let content = host.read(path).toString();
            fileChanges.sort((a, b) => b.position - a.position).forEach(change => {
                content = change.apply(content);
            });
            host.overwrite(path, content);
        }
    };
    const visitNode = (node, sourceFile) => {
        if (ts.isPropertyAccessExpression(node) &&
            ts.isIdentifier(node.name) &&
            node.name.text === oldProp) {
            const start = node.name.getStart(sourceFile);
            const end = node.name.getEnd();
            addChange(sourceFile.fileName, new util_1.FileChange(start, newProp, oldProp, 'replace'));
        }
        ts.forEachChild(node, child => visitNode(child, sourceFile));
    };
    for (const path of update.tsFiles) {
        const content = host.read(path).toString();
        const sourceFile = ts.createSourceFile(path, content, ts.ScriptTarget.Latest, true);
        visitNode(sourceFile, sourceFile);
    }
    applyChanges();
    update.applyChanges();
});
